package chat_db

import (
	"encoding/json"
	"fmt"
	"hash/fnv"
	"net/http"
	mysqlmgr "server/db/mysql"
	"server/msg"
	"strings"
	"time"
	"unicode/utf8"

	"github.com/godruoyi/go-snowflake"
)

type ChatMessage struct {
	MessageID   uint64    `gorm:"column:message_id;type:bigint unsigned;primaryKey" json:"message_id"` // 雪花ID
	SessionID   string    `gorm:"column:session_id;type:varchar(36);not null" json:"session_id"`
	SenderID    string    `gorm:"column:sender_id;type:varchar(36);not null" json:"sender_id"`
	ContentType string    `gorm:"column:content_type;type:enum('text','image','video','file');not null" json:"content_type"`
	Content     string    `gorm:"column:content;type:text;not null" json:"content"`
	CreatedAt   time.Time `gorm:"column:created_at;type:datetime(6);default:CURRENT_TIMESTAMP(6);not null" json:"created_at"`
	Status      string    `gorm:"column:status;type:enum('sending','sent','delivered','read')" json:"status"`
}

// 根据session_id哈希选择分表
func GetMessageTable(sessionID string) string {
	hash := fnv.New32a()
	hash.Write([]byte(sessionID))
	return fmt.Sprintf("chat_messages_%02d", hash.Sum32()%64)
}

// 内容类型检测
func detectContentType(content []byte) string {
	contentType := http.DetectContentType(content)
	switch {
	case strings.HasPrefix(contentType, "image/"):
		return "image"
	case strings.HasPrefix(contentType, "video/"):
		return "video"
	case strings.HasPrefix(contentType, "application/pdf"),
		strings.HasPrefix(contentType, "application/octet-stream"):
		return "file"
	case json.Valid(content):
		return "text" // JSON is treated as text in your system
	case utf8.Valid(content):
		return "text"
	default:
		return "file" // fallback to 'file' for unknown binary data
	}
}

func SendMessage(senderID, sessionID string, content []byte) (*msg.ChatMessage, error) {
	// 分布式ID生成（含时间戳）
	msgID := snowflake.ID()

	// 获取分表名
	tableName := GetMessageTable(sessionID)

	// 事务操作
	tx, _ := mysqlmgr.Begin()
	defer tx.Rollback()

	// 1. 写入消息
	_, err := tx.Exec(fmt.Sprintf(`
        INSERT INTO %s 
        (message_id, session_id, sender_id, content, content_type, status)
        VALUES (?, ?, ?, ?, ?, 'sent')`,
		tableName),
		msgID, sessionID, senderID, content, detectContentType(content),
	)
	if err != nil {
		return nil, err
	}

	// 2. 更新会话最后消息
	tx.Exec(`
        UPDATE chat_sessions 
        SET last_message_id = ?
        WHERE session_id = ?`,
		msgID, sessionID,
	)

	// 3. 更新未读计数（排除发送者自己）
	tx.Exec(`
        UPDATE session_members 
        SET unread_count = unread_count + 1
        WHERE session_id = ? AND user_id != ?`,
		sessionID, senderID,
	)

	tx.Commit()

	return &msg.ChatMessage{
		Id:        int32(msgID),
		SessionId: sessionID,
		SenderId:  senderID,
		Content:   string(content),
	}, nil
}

func GetHistoryMessages(sessionID string, lastMsgID int32, limit int) ([]*msg.ChatMessage, error) {
	tableName := GetMessageTable(sessionID)

	var query string
	var args []interface{}

	if lastMsgID <= 0 {
		// 第一次加载，查询最新的消息
		query = fmt.Sprintf(`
            SELECT message_id, sender_id, content, created_at, content_type
            FROM %s
            WHERE session_id = ?
            ORDER BY message_id DESC
            LIMIT ?`, tableName)
		args = []interface{}{sessionID, limit}
	} else {
		// 后续加载，查询比 lastMsgID 更旧的消息
		query = fmt.Sprintf(`
            SELECT message_id, sender_id, content, created_at, content_type
            FROM %s
            WHERE session_id = ? AND message_id < ?
            ORDER BY message_id DESC
            LIMIT ?`, tableName)
		args = []interface{}{sessionID, lastMsgID, limit}
	}

	rows, err := mysqlmgr.Query(query, args...)
	if err != nil {
		return nil, fmt.Errorf("GetHistoryMessages失败: %v", err)
	}
	defer rows.Close()

	var chatList []*msg.ChatMessage
	for rows.Next() {
		var chatItem ChatMessage
		err := rows.Scan(
			&chatItem.MessageID,
			&chatItem.SenderID,
			&chatItem.Content,
			&chatItem.CreatedAt,
			&chatItem.ContentType,
		)
		if err != nil {
			return nil, fmt.Errorf("解析聊天数据失败: %v", err)
		}

		chatList = append(chatList, &msg.ChatMessage{
			Id:          int32(chatItem.MessageID),
			SessionId:   chatItem.SessionID,
			SenderId:    chatItem.SenderID,
			Content:     chatItem.Content,
			CreatedAt:   chatItem.CreatedAt.String(),
			ContentType: chatItem.ContentType,
		})
	}

	return chatList, nil
}
func MarkMessagesRead(userID, sessionID string, lastReadMsgID int64) error {
	// 1. 更新已读状态
	_, err := mysqlmgr.Insert(`
        INSERT INTO message_read_status (message_id, user_id)
        SELECT message_id, ? FROM chat_messages
        WHERE session_id = ? AND message_id <= ?
        ON DUPLICATE KEY UPDATE read_at = NOW(6)`,
		userID, sessionID, lastReadMsgID,
	)

	// 2. 清零未读计数
	_, err = mysqlmgr.Update(`
        UPDATE session_members
        SET unread_count = 0
        WHERE user_id = ? AND session_id = ?`,
		userID, sessionID,
	)

	return err
}
